uniform sampler2D	uniTexture;

in vec2				varTextureCoordinates;
in vec4				varColorHSLA;


void main()
{
	float	tmpHue,
			tmpSaturation,
			tmpLightness,
			tmpOpacity,
			tmpSelector,
			tmpSegment,
			tmpInterpolator;
	int		tmpSegmentInt;
	vec4	tmpLeftHue,
			tmpRightHue,
			tmpColor,
			tmpCheckerboard;
	
	//	Read input parameters.
	tmpHue			= varColorHSLA[0];
	tmpSaturation	= varColorHSLA[1];
	tmpLightness	= varColorHSLA[2];
	tmpOpacity		= varColorHSLA[3];
	
	//	Hue

	tmpSelector		= 6.0 * tmpHue;
	tmpSegment		= floor(tmpSelector);
	tmpInterpolator	= tmpSelector - tmpSegment;
	tmpSegmentInt	= int(tmpSegment);
	
		//	My first instinct was to use an array of vec4's
		//	to look up the RGBA value as a function of the index tmpSegmentInt,
		//	but table lookups turn out to be buggy and slow (at least
		//	with GLSL 1.0 on my Radeon X1600 -- see the file
		//	"shader - uniform array.txt" for details), 
		//	so I went with the following approach instead.
		//	In effect it's mimicking the array
		//
		//			{1.0, 0.0, 0.0, 1.0},	//	red
		//			{1.0, 1.0, 0.0, 1.0},	//	yellow
		//			{0.0, 1.0, 0.0, 1.0},	//	green
		//			{0.0, 1.0, 1.0, 1.0},	//	cyan
		//			{0.0, 0.0, 1.0, 1.0},	//	blue
		//			{1.0, 0.0, 1.0, 1.0},	//	magenta
		//			{1.0, 0.0, 0.0, 1.0},	//	red
		//			{1.0, 1.0, 0.0, 1.0}	//	yellow (useful when tmpSegmentInt == 6)
		//
		//	The bool-to-float conversion send false to 0.0 and true to 1.0.

	tmpLeftHue.r	= float(tmpSegmentInt <= 1 || tmpSegmentInt >= 5);
	tmpLeftHue.g	= float(tmpSegmentInt >= 1 && tmpSegmentInt <= 3);
	tmpLeftHue.b	= float(tmpSegmentInt >= 3 && tmpSegmentInt <= 5);
	tmpLeftHue.a	= 1.0;

	tmpRightHue.r	= float(tmpSegmentInt <= 0 || tmpSegmentInt >= 4);
	tmpRightHue.g	= float(tmpSegmentInt <= 2 || tmpSegmentInt >= 6);
	tmpRightHue.b	= float(tmpSegmentInt >= 2 && tmpSegmentInt <= 4);
	tmpRightHue.a	= 1.0;

	tmpColor = mix(tmpLeftHue, tmpRightHue, tmpInterpolator);
	
	//	Saturation

	tmpColor = mix(vec4(0.5, 0.5, 0.5, 1.0), tmpColor, tmpSaturation);
	
	//	Lightness

		//	What a pleasant surprise!
		//	In some earlier testing in Starter Code, an if-else construction
		//	for deciding whether or not to include a texture
		//	slowed the program to a crawl.  So I was expecting similar
		//	results with the following if-else.  But here it works just fine!
	
	if (tmpLightness >= 0.5)
		tmpColor = mix(tmpColor, vec4(1.0, 1.0, 1.0, 1.0), 2.0*tmpLightness - 1.0);
	else
		tmpColor = mix(tmpColor, vec4(0.0, 0.0, 0.0, 1.0), 1.0 - 2.0*tmpLightness);

	//	Opacity

		//	Exagerate the opacity as α(2 - α) for greater visibility.
	tmpOpacity		= tmpOpacity * (2.0 - tmpOpacity);
	tmpCheckerboard	= texture(uniTexture, varTextureCoordinates);
	tmpColor		= mix(tmpCheckerboard, tmpColor, tmpOpacity);

	//	Write output color.
	gl_FragColor = tmpColor;
}
